
#include "unix.h"

typedef struct _Point {
    float x, y, z;
} 
Point;

typedef struct _ObjectValues {
    union {
        struct {
            float xaxis, yaxis, zaxis;
            Point center;
            Point lowerLeft, upperRight;
            char  clockwise;
        } Box;
        struct {
            Point point1, point2;
            float radius1, radius2;
            int   depth;
            char  clockwise;
            char  top, bottom;
        } Cone;
        struct {
            float xaxis, yaxis, zaxis;
            Point center;
            int   depth;
            char  clockwise;
            char  top, bottom;
        } Cylinder;
        struct {
            float xaxis, yaxis, zaxis;
            Point center;
            int   n1, n2;
            int   depth;
            char  clockwise;
        } Ellipsoid;
        struct {
            float radius;
            Point center;
            int   depth;
            char  clockwise;
        } Sphere;
        struct {
            float xaxis, yaxis, zaxis;
            float radius;
            Point center;
            int   n1, n2;
            int   depth;
            char  clockwise;
        } Torus;
        struct {
            Point  center;
            char * xy_terms;
            char * xz_terms;
            char * yz_terms;
            char * c_terms;
            double size;
            int    bounds;
        } Implicit;
        struct {
            Point  center;
            char * x_terms;
            char * y_terms;
            char * z_terms;
            char * t_limits;
            char * s_limits;
        } Parametric;
        struct {
            int    seed;
            int    maxlevel;
            double H;
            double sigma;
            int    addition;
        } Mountain;
    } Obj;
} ObjectStruct;


#ifdef False
#undef False
#endif
#define False    0

#ifdef True
#undef True
#endif
#define True    1

#define CLOCKWISE            0
#define COUNTERCLOCKWISE    1

#define ZERO_TOLERANCE    1.0e-6

#define ABS(_x)            ((_x) < 0.0 ? -(_x) : (_x))
#define NEAR_ZERO(_s)    (ABS (_s) < ZERO_TOLERANCE)
#define round(x) (((x)>=0.0) ? ((int)((x)+0.5)) : ((int)((x)-0.5)))


/* Dot product of v1 * v2 */
#define DOT_PRODUCT(_v1, _v2) \
    ((_v1)->x * (_v2)->x + (_v1)->y * (_v2)->y + (_v1)->z * (_v2)->z)


/* Cross product of p1p2 x p3p4 */
#define CROSS_PRODUCT(_p1, _p2, _p3, _p4, _v) \
{ \
    (_v)->x = (((_p2)->y - (_p1)->y) * ((_p4)->z - (_p3)->z)) - \
              (((_p2)->z - (_p1)->z) * ((_p4)->y - (_p3)->y)); \
    (_v)->y = (((_p2)->z - (_p1)->z) * ((_p4)->x - (_p3)->x)) - \
              (((_p2)->x - (_p1)->x) * ((_p4)->z - (_p3)->z)); \
    (_v)->z = (((_p2)->x - (_p1)->x) * ((_p4)->y - (_p3)->y)) - \
              (((_p2)->y - (_p1)->y) * ((_p4)->x - (_p3)->x)); \
}


/* Normalize vector and return length */
#define NORMALIZE_VECTOR(_v, _len) \
{ \
    _len = DOT_PRODUCT (_v, _v); \
    _len = sqrt (_len); \
    if (!NEAR_ZERO (_len)) \
    { \
      (_v)->x /= _len; \
      (_v)->y /= _len; \
      (_v)->z /= _len; \
    } \
}
