
#include "unix.h"
#include "x.h"
#include "xt.h"
#include "xmu.h"
#include "xpex.h"
#include "color.h"
#include "externs.h"
#include "globals.h"

extern XtAppContext app_context;
extern Display     *display;
extern GC           gc;
extern Visual      *visual;
extern int          depth;
extern Colormap     colormap;
       XStandardColormap   cmap_info;
	   PEXColorApproxEntry approxEnt;
extern Colormap     cmap;
       Cursor       cursor;
extern XColor       colors[64];
extern GC  boxedGC, boxedChangedGC, unboxedGC;

extern int bars_locked;
extern Boolean do_change;
extern Pixel original_background;
extern float pass_value;

extern Widget toplevel, mixingForm, mixedColor;
extern Widget rgb, hsv, hls, cie;

extern Widget hsvColor;

extern Widget title;
extern Widget tripleValue;
extern Widget hueLocked, satLocked, valLocked;
extern Widget hueScroll, satScroll, valScroll;
extern Widget hueLabel, satLabel, valLabel;
extern Widget quit;

extern Widget redLocked, greenLocked, blueLocked;
extern Widget redLabel, greenLabel, blueLabel;
extern Widget redScroll, greenScroll, blueScroll, lockedScroll;

extern Boolean workingRGB, workingHSV, workingHLS, workingCIE;

extern XtActionProc hsv_BtnDn(), hsv_Configure(), hsv_Expose(), hsv_Enter();

static XtActionsRec hsv_actions[] =
{
   { "enter",     (XtActionProc)hsv_Enter },
   { "btnDn",     (XtActionProc)hsv_BtnDn },
   { "configure", (XtActionProc)hsv_Configure },
   { "expose",    (XtActionProc)hsv_Expose }
};


static String hsv_translations =
  "<EnterWindow>: enter()\n\
   <BtnDown>:     btnDn()\n\
   <Configure>:   configure()\n\
   <Expose>:      expose()";


void makeHSV_Color( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    Widget popup;
    XGCValues values;
    extern PEXColorSpecifier colorSpec;
    int type;
    XVisualInfo       visualInfo;
    PEXExtensionInfo *pexinfo;
    char              err[80];

    /* See if a PEX extension exists and initialize PEXlib. */
    if ( PEXInitialize( display, &pexinfo, 80, err ) != 0 ) {
        fprintf( stderr, "No PEX extension.\n" );
        fprintf( stderr, "%s\n", err );
        exit(1);
    }

#ifndef PSEUDO
    /* Determine the best visual to use. */
    type = ora_find_best_visual( display, &visualInfo );
    /* If type == PEXTrue after this statement */
    /* then pixmap drawable is available.   */
    depth = visualInfo.depth;
    visual = visualInfo.visual;
#else
    depth = DefaultDepth( display, DefaultScreen( display ) );
    visual = DefaultVisual( display, DefaultScreen( display ) );
#endif

    if( depth != 8 )
        ora_create_colormap( display, &visualInfo, &cmap_info, &approxEnt );
    else {
        /* Get a standard colormap for the visual. */
        if ( !ora_get_standard_colormap( display, &visualInfo, &cmap_info ) ) {
            fprintf ( stderr, "Cannot find a standard colormap\n" );
            exit(1);
        }
        ora_set_stdcmap_approx( &visualInfo, &cmap_info, &approxEnt );
    }

    colormap = cmap_info.colormap;

    XtSetSensitive( rgb, False );    workingRGB = False;
    XtSetSensitive( hsv, False );    workingHSV = True;
                                     colorSpec.type = PEXColorTypeHSV;
    XtSetSensitive( hls, False );    workingHLS = False;
    XtSetSensitive( cie, False );    workingCIE = False;

    popup = XtVaCreateManagedWidget( "ColorEditor",
        topLevelShellWidgetClass, toplevel,
        XtNwidth, 425,
        XtNheight, 353,
        XtNvisual, visual,
        XtNdepth, depth,
        XtNcolormap, colormap,
        NULL);
    mixingForm = XtVaCreateManagedWidget("mixingForm", formWidgetClass, popup,
        NULL);

    title = XtVaCreateManagedWidget("title", labelWidgetClass, mixingForm,
        XtNlabel, "Choose HSV Color",
        NULL);

    cursor = XCreateFontCursor( display, XC_dotbox );
    hsvColor = XtVaCreateManagedWidget("hsvColor", labelWidgetClass,
        mixingForm,
        XtNfromVert, title,
        XtNheight, 200,
        XtNwidth, 200,
        XtNlabel, "",
        XtNcursor, cursor,
        NULL);
    mixedColor = XtVaCreateManagedWidget("mixedColor", labelWidgetClass,
        mixingForm,
        XtNlabel, "",
        XtNheight, 200,
        XtNwidth, 90,
        XtNbackground, colors[MIX_COLOR].pixel,
        XtNfromVert, title,
        XtNfromHoriz, hsvColor,
        NULL);

    tripleValue = XtVaCreateManagedWidget("tripleValue", toggleWidgetClass,
        mixingForm,
        XtNfromHoriz, hsvColor,
        XtNwidth, 90,
        XtNheight, 23,
        NULL);

    hueLocked = XtVaCreateManagedWidget("hueLocked", labelWidgetClass,
        mixingForm,
        XtNfromVert, hsvColor,
        NULL);
    hueScroll = XtVaCreateManagedWidget("hueScroll", scrollbarWidgetClass,
        mixingForm,
        XtNfromVert, hsvColor,
        XtNfromHoriz, hueLocked,
        NULL);
    hueLabel = XtVaCreateManagedWidget("hueLabel", labelWidgetClass, mixingForm,
        XtNfromVert, hsvColor,
        XtNfromHoriz, hueScroll,
        NULL);

    satLocked = XtVaCreateManagedWidget("satLocked", labelWidgetClass,
        mixingForm,
        XtNfromVert, hueLocked,
        NULL);
    satScroll = XtVaCreateManagedWidget("satScroll", scrollbarWidgetClass,
        mixingForm,
        XtNfromVert, hueLocked,
        XtNfromHoriz, satLocked,
        NULL);
    satLabel = XtVaCreateManagedWidget("satLabel", labelWidgetClass, mixingForm,
        XtNfromVert, hueLocked,
        XtNfromHoriz, satScroll,
        NULL);

    valLocked = XtVaCreateManagedWidget("valLocked", labelWidgetClass,
        mixingForm,
        XtNfromVert, satLocked,
        NULL);
    valScroll = XtVaCreateManagedWidget("valScroll", scrollbarWidgetClass,
        mixingForm,
        XtNfromVert, satLocked,
        XtNfromHoriz, valLocked,
        NULL);
    valLabel = XtVaCreateManagedWidget("valLabel", labelWidgetClass, mixingForm,
        XtNfromVert, satLocked,
        XtNfromHoriz, valScroll,
        NULL);

    redLocked = XtVaCreateManagedWidget( "redLocked", labelWidgetClass,
        mixingForm, NULL );
    greenLocked = XtVaCreateManagedWidget( "greenLocked", labelWidgetClass,
        mixingForm,
        NULL );
    blueLocked = XtVaCreateManagedWidget( "blueLocked", labelWidgetClass,
        mixingForm, NULL );

    cursor = XCreateFontCursor( display, XC_sb_v_double_arrow );

    redScroll = XtVaCreateManagedWidget( "redScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );
    greenScroll = XtVaCreateManagedWidget( "greenScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );
    blueScroll = XtVaCreateManagedWidget( "blueScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );
    lockedScroll = XtVaCreateManagedWidget("lockedScroll", scrollbarWidgetClass,
        mixingForm,
        XtNscrollVCursor, cursor,
        NULL );

    redLabel = XtVaCreateManagedWidget( "redLabel", labelWidgetClass,
        mixingForm,
        XtNfromHoriz, mixedColor,
        XtNfromVert, mixedColor,
        NULL );
    greenLabel = XtVaCreateManagedWidget( "greenLabel", labelWidgetClass,
        mixingForm,
        XtNfromHoriz, redScroll,
        XtNfromVert, redLabel,
        NULL );
    blueLabel = XtVaCreateManagedWidget( "blueLabel", labelWidgetClass,
        mixingForm,
        XtNfromHoriz, greenScroll,
        XtNfromVert, greenLabel,
        NULL );

    quit = XtVaCreateManagedWidget("quit", commandWidgetClass, mixingForm,
        XtNfromVert, valScroll,
        NULL);

    values.foreground = colors[PURE_RED].pixel;
    values.line_width = 2;
    values.line_style = LineOnOffDash;
    boxedChangedGC = XtGetGC(mixingForm, GCForeground | GCLineWidth,
        &values);
    boxedGC = XtGetGC(mixingForm, GCForeground | GCLineWidth | GCLineStyle,
        &values);

    XtVaGetValues(mixingForm, XtNbackground, &(values.foreground), NULL);
    unboxedGC = XtGetGC(mixingForm, GCForeground | GCLineWidth, &values);

    original_background = values.foreground;

    bars_locked = 0;

    XtAddCallback(hueScroll, XtNscrollProc, Scrolled, (XtPointer)HUE);
    XtAddCallback(satScroll, XtNscrollProc, Scrolled, (XtPointer)SAT);
    XtAddCallback(valScroll, XtNscrollProc, Scrolled, (XtPointer)VAL);

    XtAddCallback(hueScroll, XtNjumpProc, Update_HSV, (XtPointer)HUE);
    XtAddCallback(satScroll, XtNjumpProc, Update_HSV, (XtPointer)SAT);
    XtAddCallback(valScroll, XtNjumpProc, Update_HSV, (XtPointer)VAL);

    XtAddCallback( redScroll, XtNscrollProc, Scrolled, ( XtPointer )RED );
    XtAddCallback( greenScroll, XtNscrollProc, Scrolled, ( XtPointer )GREEN );
    XtAddCallback( blueScroll, XtNscrollProc, Scrolled, ( XtPointer )BLUE );
    XtAddCallback( lockedScroll, XtNscrollProc, Scrolled, ( XtPointer )LOCKED );

    XtAddCallback( redScroll, XtNjumpProc, Thumbed, ( XtPointer )RED );
    XtAddCallback( greenScroll, XtNjumpProc, Thumbed, ( XtPointer )GREEN );
    XtAddCallback( blueScroll, XtNjumpProc, Thumbed, ( XtPointer )BLUE );
    XtAddCallback( lockedScroll, XtNjumpProc, Thumbed, ( XtPointer )LOCKED );

    XtAddCallback(tripleValue, XtNcallback, Set_Selection,
        (XtPointer)PRIMARY_COLOR);
    XtAddCallback(quit, XtNcallback, Quit, (XtPointer)toplevel);

    XtAppAddActions( app_context, hsv_actions, XtNumber( hsv_actions ));
    XtOverrideTranslations( hsvColor,
        XtParseTranslationTable( hsv_translations ));

    /* set the scrollbars to the initial mixed colour values */
    do_change = FALSE;
    CHANGE_HSV_HUE( );
    CHANGE_HSV_SAT( );
    CHANGE_HSV_VAL( );
    CHANGE_RED( MIX_COLOR );
    CHANGE_GREEN( MIX_COLOR );
    CHANGE_BLUE( MIX_COLOR );
    do_change = TRUE;
    XStoreColors(display, cmap, colors, 4);

    XtRealizeWidget( popup );
    XtPopup( popup, XtGrabNone );
}
