/*	@(#)pexutdbint.h 1.1 94/05/27 SMI	*/

/* $Header: pexutdbint.h,v 1.1 93/03/02 16:39:51 waitz Exp $ */

/******************************************************************************/
/*  (c) Copyright Hewlett-Packard Company, 1992,  Fort Collins, Colorado      */
/*                                                                            */
/*                            All Rights Reserved                             */
/*                                                                            */
/*  Permission to use, copy, modify, and distribute this software and its     */
/*  documentation for any purpose and without fee is hereby granted provided  */
/*  that the above copyright notices appear in all copies and that both the   */
/*  copyright notices and this permission notice appear in supporting         */
/*  documentation, and that the name of Hewlett-Packard not be used in        */
/*  advertising or publicity pertaining to distribution of the software       */
/*  without specific, written prior permission.                               */
/*                                                                            */
/*  HEWLETT-PACKARD MAKES NO WARRANTY OF ANY KIND WITH REGARD TO THIS         */
/*  SOFTWARE, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  Hewlett-Packard    */
/*  shall not be liable for errors contained herein or direct, indirect,      */
/*  special, incidental or consequential damages in connection with the       */
/*  furnishing, performance or use of this software.                          */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/*                                                                            */
/* Module:  PEXlib Double-Buffer Utilities (see file pexutdb.h for more info) */
/*                                                                            */
/* File Content:                                                              */
/*   This file contains constants, data structures and function declarations  */
/*   for internal use in the PEXlib double-buffer utilities.                  */
/*                                                                            */
/******************************************************************************/


#ifndef _PEXUTDBINT_H_ /* [ */
#define _PEXUTDBINT_H_


#ifndef NeedFunctionPrototypes
#if defined(FUNCPROTO) || defined(__STDC__) || \
    defined(__cplusplus) || defined(c_plusplus)
#define NeedFunctionPrototypes  1
#else
#define NeedFunctionPrototypes  0
#endif /* FUNCPROTO, __STDC__, __cplusplus, c_plusplus */
#endif /* NeedFunctionPrototypes */


#ifdef __cplusplus   /* do not leave open across includes */
extern "C" {         /* for C++ V2.0 */
#endif

#include <X11/extensions/multibuf.h>

#include <X11/PEX5/ESproto.h>
#include <X11/PEX5/HPproto.h>

/*
** constants for dbtype variable in PEXUtDBData
*/
#define PEXUTDBMBX      0
#define PEXUTDBESC      1
#define PEXUTDBPIX      2

/*
** Global internal data
*/
typedef struct pexutdbdata {
    Window              window;         /* window id - also search key        */
    Display             *display;       /* display ptr - secondary search key */
    struct pexutdbdata  *next;          /* ptr to next node in the data list  */
    int                 db_type;        /* indicates db mechanism used        */
    int                 front_supported;/* indicates support of render front  */
    int                 render_to_front;/* flag indicating front or back      */
    int                 db_x;           /* indicates double-buffer X rendering*/
    Multibuffer         back_buffer;    /* current back buffer, if MBX        */
    Multibuffer         buffers[2];     /* buffer identifiers, if using MBX   */
    PEXRenderer         renderer;       /* renderer id, used for clearing     */
    int                 clear_renderer; /* flag indicating whether to clear   */
    Pixmap              pixmap;         /* pixmap identifier, if using pixmap */
    GC                  gc;             /* gc for clearing pixmap             */
    int                 width;          /* window width                       */
    int                 height;         /* window height                      */
    int                 depth;          /* window depth                       */
} PEXUtDBData;

#define HP_ESCAPE_SETRENDERINGBUFFER            HP_ESCAPE_DFRONT
#define HP_ESCAPE_ET_SETRENDERINGBUFFER         HP_ESCAPE_ET_DFRONT
#define HP_ESCAPE_ETM_SETRENDERINGBUFFER        HP_ESCAPE_ETM_DFRONT

/******************************************************************************/
/*                                                                            */
/* Internal Functions Description:                                            */
/*   This set of functions provides a simple interface for managing global    */
/*   data in the PEXlib double-buffer utilities.                              */
/*                                                                            */
/* Internal Functions:                                                        */
/*   lookup_node  - lookup the data node for the specified window             */
/*   add_node     - add a new data node for the specified window              */
/*   delete_node  - delete the data node for the specified window             */
/*                                                                            */
/******************************************************************************/

/*
** function declarations
*/

/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/*   lookup_node    - lookup the data node for the specified window           */
/*                                                                            */
/* Returns:                                                                   */
/*   PEXUtDBData *  - a pointer to the data node                              */
/*                                                                            */
/* Arguments:                                                                 */
/*   win            - window id search key                                    */
/*   dpy            - display ptr secondary search key                        */
/*                                                                            */
/******************************************************************************/
extern PEXUtDBData *lookup_node(
#if NeedFunctionPrototypes
    Window              /* win */,
    Display *           /* dpy */
#endif
);

/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/*   add_node       - add a new data node for the specified window            */
/*                                                                            */
/* Returns:                                                                   */
/*   PEXUtDBData *  - a pointer to the new data node                          */
/*                                                                            */
/* Arguments:                                                                 */
/*   win            - window id search key                                    */
/*   dpy            - display ptr secondary search key                        */
/*                                                                            */
/******************************************************************************/
extern PEXUtDBData *add_node(
#if NeedFunctionPrototypes
    Window              /* win */,
    Display *           /* dpy */
#endif
);

/******************************************************************************/
/*                                                                            */
/* Function:                                                                  */
/*   delete_node    - delete the data node for the specified window           */
/*                                                                            */
/* Returns:                                                                   */
/*   void           - no return value                                         */
/*                                                                            */
/* Arguments:                                                                 */
/*   win            - window id search key                                    */
/*   dpy            - display ptr secondary search key                        */
/*                                                                            */
/******************************************************************************/
extern void delete_node(
#if NeedFunctionPrototypes
    Window              /* win */,
    Display *           /* dpy */
#endif
);


/*
** macro definition
*/

#define LOOKUP_NODE(p,w,d)                                                     \
    if ( ( pexutdb_last != (PEXUtDBData *)NULL ) &&                            \
         ( ( pexutdb_last->window == w ) && ( pexutdb_last->display == d ) ) ) \
        p = pexutdb_last;                                                      \
    else {                                                                     \
        p = lookup_node( w, d );                                               \
        pexutdb_last = p;                                                      \
    }


#ifdef __cplusplus
}                    /* for C++ V2.0 */
#endif


#endif /* _PEXUTDBINT_H_ ] */
