
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xm/Separator.h>
#include <Xm/PushB.h>
#include <Xm/CascadeB.h>
#include <Xm/RowColumn.h>
#include <Xm/Label.h>

typedef struct _menu_struct{
   char*                  name;           /* name of the button */
   void                  (*func)();       /* Callback to be invoked */
   caddr_t                data;           /* Data for the callback */
   struct _menu_struct   *sub_menu;       /* data for submenu of this button */
   int                    n_sub_items;    /* How many items in sub_menu */
   char                  *sub_menu_title; /* Title of submenu */
} menu_struct;

void
create_menus(title, menu, menulist, nitems, backPix)
char        *title;
Widget       menu;
menu_struct *menulist;
int          nitems;
Pixmap       backPix;
{
    Arg        wargs[1];
    int        i;
    WidgetList buttons;
    int        separators = 0;
    /*
     * Allocate a widget list to hold all 
     * button widgets.
     */
    buttons = (WidgetList) malloc(nitems * sizeof(Widget));
    /*
     * If a title is given, create Label and Separator widgets.
     */
    if(title){
        XtCreateManagedWidget(title, xmLabelWidgetClass, menu,
            NULL, 0);
        XtCreateManagedWidget("separator", xmSeparatorWidgetClass,
            menu, NULL, 0);
    }
    /*
     * Create an entry for each item in the menu.
     */
    for(i=0;i<nitems;i++){
        /* 
         * A NULL name represents a separator.
         */
        if(menulist[i].name == NULL){
            XtCreateManagedWidget("separator",
                xmSeparatorWidgetClass,
                menu, NULL, 0);
            separators++; /* Count how many entries aren't buttons */
        }
        /*
         * If there is a name and a callback, create a "normal"
         * menu entry and register the callback function.
         */
        else if(menulist[i].func){
            buttons[i-separators] = XtCreateWidget(menulist[i].name,
                xmPushButtonWidgetClass,
                menu, NULL, 0);
            XtAddCallback(buttons[i-separators], XmNactivateCallback,
                menulist[i].func, menulist[i].data);
        }
        /*
         * If there is a name, but no callback function, the entry
         * must be a label, unless there is a submenu.
         */
        else if(!menulist[i].sub_menu){
            buttons[i-separators] = XtCreateWidget(menulist[i].name,
                xmLabelWidgetClass,
                menu, NULL, 0);
        }
        /*
         * If we got here, the entry must be a submenu.
         * Create a pulldown menu pane and an XmCascadeButton
         * widget. Attach the menu pane and make a recursive call
         * to create the entries in the submenu.
         */
        else{
            Widget sub_menu;
            sub_menu =XmCreatePulldownMenu(menu,
                menulist[i].sub_menu_title,
                NULL, 0);
            XtSetArg(wargs[0], XmNsubMenuId, sub_menu);
            buttons[i-separators] =
                XtCreateWidget(menulist[i].name,
                xmCascadeButtonWidgetClass,
                menu, wargs, 1);
            create_menus(menulist[i].sub_menu_title,
                sub_menu, menulist[i].sub_menu,
                menulist[i].n_sub_items);
        }
    }
    /*
     * Manage all button widgets. Menu panes are not managed.
     */
    XtManageChildren(buttons, nitems - separators);
}
